{{         ---------  packet comm  --- January 2015 --- By Ray Tracy  ----------
    This implements serial packet based comm between my Raspberry PI and P8X32A Propeller
  Packet size is limited to 256 bytes in this implementation but is easily changed to any
  size needed.  The contents of the packet  are unrestricted.
     I have only the RPI and the Prop connected and therefore have not implemented any
  addressing scheme. I use byte zero from the RPI as the command or packet type and the
  remaining bytes in the packet are determined by that command. Routines are provided that
  convert bytes to and from propeller longs as well as inserting and extracting strings
  from the packet.

 --------------------------------------------------------------------------------
  TODO:  add timeout/abort to waitsoh to prevent lockups
}}
CON 
  _clkmode  = xtal1 + pll16x
  _xinfreq  = 5_000_000
  Clock    = (_CLKMODE >> 6) * _XINFREQ         ' clock in ticks per second as a constant 
  Ms1      = Clock / 1000

' ======<< Ascii control Characters >>=====
   soh = %00000001   '01, $01, %00000001
   stx = %00000010   '02, $02, %00000010
   etx = %00000011   '03, $03, %00000011
   eot = %00000100   '04, $04, %00000100
   ack = %00000110   '06, $06, %00000110
   nak = %00010101   '21, $15, %00010101
   syn = %00010110   '22, $16, %00010110
   

' ==============<< Packet Comm Port Assignments >>================
  rxPin              = 31
  txPin              = 30
  PcomBaud           = 57600           ' Limited by Raspberry PI
  #0,Term,Rnxv,Pcom

  BufferSize = 64                      ' Max, Adjust this for the size you need
  #0, Left, Right
  #0, Low, High
  
OBJ                              ' Main cog
  Ser    : "FourPortSerial"      ' Uses 1 cog  Modified from OBEX
   
VAR
   byte PktSize
   byte SndPkt[BufferSize]                 
   byte RcvPkt[BufferSize]
   byte Ck1, c1, c2
   
PUB Init
    Ser.AddPort(Pcom, rxPin, txPin,-1,-1,0, 0, PcomBaud)   'Port,Rx,Tx,Cts,Rts,Threshold,Mode,Baud


Pri Pause(Miliseconds)
   waitcnt(Miliseconds*ms1+cnt)

{------------------------<< SEND RECEIVE PACKET >>----------------------------------------
                 *****   MY PACKET DEFINITION   *****
     Each packet begins with packet start sequence followed by the <count> followed by the
     data bytes and the <checksum>. There are no restrictions on the contents of the data.
     The checksum is the mod 256 sum of the data not including soh, stx, count or cksum.
     <packet>      ::= <packetstart><count><data><cksum>
     <packetstart> ::= soh stx
     <count>       ::= $00..BufferSize
     <cksum>       ::= ( Sum<data> ) mod 256
     <data>        ::= <byte> | <data><byte>
     <byte>        ::= $00..$FF

 ------------------------------------------------------------------------------------------}
PUB RcvPacket( Pkt )| i, N
    Waitsoh
    N := Ser.rx(Pcom)                      ' Get the packet size
    Ck1 := 0                               ' Clear the counter and checksum  
    I := 0
    repeat                 
        RcvPkt[I] := Ser.rx(Pcom)          ' Get packet data
        Ck1 += RcvPkt[I]                   ' Recompute the checksum
        I++
    while (i < N)          
    c1 :=  Ser.rx(Pcom)                    ' Get remote checksum
    Ck1 //= 256                            ' Finish checksum calculation
    if ( c1 == Ck1 )                       ' Do they Match
        return N                           
    return 0                               '     No - Arghhh

PUB SndPacket( N, Pkt ) | I
     Ser.tx(Pcom,soh)                       ' Send start of Transmission
     Ser.tx(Pcom,stx)                       ' Send start of Packet
     Ser.tx(Pcom,N)                         ' Send the Packet size
     Ck1 := 0                               ' Clear counters and checksum
     I := 0
     repeat                                
        Ck1 += SndPkt[I]                    ' Compute running checksum
        Ser.tx(Pcom,SndPkt[I])              ' Sent the next data byte
        I++
     while (I < N)
     Ck1 //= 256                            ' Finish the checksum calculation
     Ser.tx(Pcom, Ck1 )                     ' Send the checksum

Pri Waitsoh
    c2 := 0                                 ' initialize
    repeat
       c1 := c2                             ' Save as first byte
       c2 := Ser.rx(Pcom)                   ' Get second byte
    until (( c1==soh) and (c2==stx))        ' Return if start sequence detected

PUB AckNak(tf)
   case tf
     True : SndPkt[0] := ack
     False: SndPkt[0] := nak
   SndPacket(1,SndPkt)
 
{------------------------------------------------------------------------
  These are utility routines used to move data from and into the packet
 ------------------------------------------------------------------------}
Pub Pack(N,i,Size)| j                   ' Packs N for Size bytes into SndPkt beginning at i
   repeat j from 0 to Size-1            ' Can be used for Words, Longs, Strings.
      SndPkt[i] := N.Byte[j]
      I++

Pub UnPack( i, size):N | j              ' Unpacks Size bytes from RcvPkt beginning at i
   repeat j from 0 to Size-1
      N.byte[j] := RcvPkt[i]
      I++

Pub StrCp(src,dst)                     ' Copies src to dst for sizeof(src)
   bytemove(dst,src,strsize(src)+1)    ' Plus one includes the string terminator
   return strsize(src)+1               ' return the size of inserted string

Pub ClearPacket(dst,Size)              ' Clears dst to all zeros
   bytefill(dst,0,Size)

Pri BuildPacket(N) | i                      ' This will build a dummy packet
    repeat i from 0 to N
       SndPkt[i] := (i)
     
